package com.sun.xml.internal.ws.api.streaming;

import java.lang.reflect.InvocationTargetException;
import javax.xml.stream.XMLStreamException;
import java.io.StringReader;
import java.lang.reflect.Method;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.io.UnsupportedEncodingException;
import java.io.InputStreamReader;
import com.sun.istack.internal.NotNull;
import java.io.Reader;
import java.io.InputStream;
import com.sun.istack.internal.Nullable;
import java.io.IOException;
import com.sun.xml.internal.ws.streaming.XMLReaderException;
import java.net.URL;
import javax.xml.stream.XMLStreamReader;
import org.xml.sax.InputSource;
import com.sun.xml.internal.ws.util.xml.XmlUtil;
import com.sun.xml.internal.ws.resources.StreamingMessages;
import java.util.logging.Level;
import javax.xml.stream.XMLInputFactory;
import java.util.logging.Logger;

public abstract class XMLStreamReaderFactory
{
    private static final Logger LOGGER;
    private static final String CLASS_NAME_OF_WSTXINPUTFACTORY = "com.ctc.wstx.stax.WstxInputFactory";
    private static volatile ContextClassloaderLocal<XMLStreamReaderFactory> streamReader;
    
    public XMLStreamReaderFactory() {
    }
    
    private static XMLInputFactory getXMLInputFactory() {
        XMLInputFactory xif = null;
        if ((boolean)getProperty(XMLStreamReaderFactory.class.getName() + ".woodstox")) {
            try {
                xif = (XMLInputFactory)Class.forName("com.ctc.wstx.stax.WstxInputFactory").newInstance();
            }
            catch (Exception e) {
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.WARNING)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.WARNING, StreamingMessages.WOODSTOX_CANT_LOAD("com.ctc.wstx.stax.WstxInputFactory"), e);
                }
            }
        }
        if (xif == null) {
            xif = XmlUtil.newXMLInputFactory(true);
        }
        xif.setProperty("javax.xml.stream.isNamespaceAware", Boolean.valueOf(true));
        xif.setProperty("javax.xml.stream.supportDTD", Boolean.valueOf(false));
        xif.setProperty("javax.xml.stream.isCoalescing", Boolean.valueOf(true));
        return xif;
    }
    
    public static void set(XMLStreamReaderFactory f) {
        if (f == null) {
            throw new IllegalArgumentException();
        }
        XMLStreamReaderFactory.streamReader.set(f);
        return;
    }
    
    public static XMLStreamReaderFactory get() {
        return (XMLStreamReaderFactory)XMLStreamReaderFactory.streamReader.get();
    }
    
    public static XMLStreamReader create(InputSource source, boolean rejectDTDs) {
        try {
            if (source.getCharacterStream() != null) {
                return get().doCreate(source.getSystemId(), source.getCharacterStream(), rejectDTDs);
            }
            if (source.getByteStream() != null) {
                return get().doCreate(source.getSystemId(), source.getByteStream(), rejectDTDs);
            }
            return get().doCreate(source.getSystemId(), new URL(source.getSystemId()).openStream(), rejectDTDs);
        }
        catch (IOException e) {
            throw new XMLReaderException("stax.cantCreate", new Object[] { e });
        }
    }
    
    public static XMLStreamReader create(@Nullable String systemId, InputStream in, boolean rejectDTDs) {
        return get().doCreate(systemId, in, rejectDTDs);
    }
    
    public static XMLStreamReader create(@Nullable String systemId, InputStream in, @Nullable String encoding, boolean rejectDTDs) {
        return (encoding == null) ? create(systemId, in, rejectDTDs) : get().doCreate(systemId, in, encoding, rejectDTDs);
    }
    
    public static XMLStreamReader create(@Nullable String systemId, Reader reader, boolean rejectDTDs) {
        return get().doCreate(systemId, reader, rejectDTDs);
    }
    
    public static void recycle(XMLStreamReader r) {
        get().doRecycle(r);
        if ((r instanceof XMLStreamReaderFactory.RecycleAware)) {
            ((XMLStreamReaderFactory.RecycleAware)r).onRecycled();
        }
    }
    
    public abstract XMLStreamReader doCreate(String p0, InputStream p1, boolean p2);
    
    private XMLStreamReader doCreate(String systemId, InputStream in, @NotNull String encoding, boolean rejectDTDs) {
        Reader reader;
        try {
            reader = new InputStreamReader(in, encoding);
        }
        catch (UnsupportedEncodingException ue) {
            throw new XMLReaderException("stax.cantCreate", new Object[] { ue });
        }
        return this.doCreate(systemId, reader, rejectDTDs);
    }
    
    public abstract XMLStreamReader doCreate(String p0, Reader p1, boolean p2);
    
    public abstract void doRecycle(XMLStreamReader p0);
    
    private static int buildIntegerValue(String propertyName, int defaultValue) {
        String propVal = System.getProperty(propertyName);
        if (propVal != null && propVal.length() > 0) {
            try {
                Integer value = Integer.valueOf(Integer.parseInt(propVal));
                if ((int)value > 0) {
                    return (int)value;
                }
            }
            catch (NumberFormatException nfe) {
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.WARNING)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.WARNING, StreamingMessages.INVALID_PROPERTY_VALUE_INTEGER(propertyName, propVal, Integer.toString(defaultValue)), nfe);
                }
            }
        }
        return defaultValue;
    }
    
    private static long buildLongValue(String propertyName, long defaultValue) {
        String propVal = System.getProperty(propertyName);
        if (propVal != null && propVal.length() > 0) {
            try {
                long value = Long.parseLong(propVal);
                if (value > 0L) {
                    return value;
                }
            }
            catch (NumberFormatException nfe) {
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.WARNING)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.WARNING, StreamingMessages.INVALID_PROPERTY_VALUE_LONG(propertyName, propVal, Long.toString(defaultValue)), nfe);
                }
            }
        }
        return defaultValue;
    }
    
    private static Boolean getProperty(final String prop) {
        return (Boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                String value = System.getProperty(prop);
                return (value != null) ? Boolean.valueOf(value) : Boolean.FALSE;
            }
        });
    }
    
    static XMLInputFactory access$000() {
        return getXMLInputFactory();
    }
    
    static Boolean access$100(String x0) {
        return getProperty(x0);
    }
    
    static int access$300(String x0, int x1) {
        return buildIntegerValue(x0, x1);
    }
    
    static long access$400(String x0, long x1) {
        return buildLongValue(x0, x1);
    }
    
    static {
        LOGGER = Logger.getLogger(XMLStreamReaderFactory.class.getName());
        XMLStreamReaderFactory.streamReader = new ContextClassloaderLocal<XMLStreamReaderFactory>() {
            @Override
            protected XMLStreamReaderFactory initialValue() {
                XMLInputFactory xif = XMLStreamReaderFactory.access$000();
                XMLStreamReaderFactory f = null;
                if (!(boolean)XMLStreamReaderFactory.access$100(XMLStreamReaderFactory.class.getName() + ".noPool")) {
                    f = XMLStreamReaderFactory.Zephyr.newInstance(xif);
                }
                if (f == null && xif.getClass().getName().equals("com.ctc.wstx.stax.WstxInputFactory")) {
                    f = new Woodstox(xif);
                }
                if (f == null) {
                    f = new XMLStreamReaderFactory.Default();
                }
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.FINE, "XMLStreamReaderFactory instance is = {0}", f);
                    return f;
                }
                return f;
            }
        };
    }
    
    private static final class Zephyr extends XMLStreamReaderFactory
    {
        private final XMLInputFactory xif;
        private final ThreadLocal<XMLStreamReader> pool;
        private final Method setInputSourceMethod;
        private final Method resetMethod;
        private final Class zephyrClass;
        
        @Nullable
        public static XMLStreamReaderFactory newInstance(XMLInputFactory xif) {
            try {
                Class<?> clazz = xif.createXMLStreamReader(new StringReader("<foo/>")).getClass();
                if (!clazz.getName().startsWith("com.sun.xml.internal.stream.")) {
                    return null;
                }
                return new XMLStreamReaderFactory.Zephyr(xif, clazz);
            }
            catch (NoSuchMethodException e) {
                return null;
            }
            catch (XMLStreamException e2) {
                return null;
            }
        }
        
        public Zephyr(XMLInputFactory xif, Class clazz) throws NoSuchMethodException {
            this.pool = new ThreadLocal();
            this.zephyrClass = clazz;
            this.setInputSourceMethod = clazz.getMethod("setInputSource", new Class[] { InputSource.class });
            this.resetMethod = clazz.getMethod("reset", new Class[0]);
            try {
                xif.setProperty("reuse-instance", Boolean.valueOf(false));
            }
            catch (IllegalArgumentException ex) {}
            this.xif = xif;
        }
        
        @Nullable
        private XMLStreamReader fetch() {
            XMLStreamReader sr = (XMLStreamReader)this.pool.get();
            if (sr == null) {
                return null;
            }
            this.pool.set(null);
            return sr;
        }
        
        @Override
        public void doRecycle(XMLStreamReader r) {
            if (this.zephyrClass.isInstance(r)) {
                this.pool.set(r);
            }
        }
        
        @Override
        public XMLStreamReader doCreate(String systemId, InputStream in, boolean rejectDTDs) {
            try {
                XMLStreamReader xsr = this.fetch();
                if (xsr == null) {
                    return this.xif.createXMLStreamReader(systemId, in);
                }
                InputSource is = new InputSource(systemId);
                is.setByteStream(in);
                this.reuse(xsr, is);
                return xsr;
            }
            catch (IllegalAccessException e) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e });
            }
            catch (InvocationTargetException e2) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e2 });
            }
            catch (XMLStreamException e3) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e3 });
            }
        }
        
        @Override
        public XMLStreamReader doCreate(String systemId, Reader in, boolean rejectDTDs) {
            try {
                XMLStreamReader xsr = this.fetch();
                if (xsr == null) {
                    return this.xif.createXMLStreamReader(systemId, in);
                }
                InputSource is = new InputSource(systemId);
                is.setCharacterStream(in);
                this.reuse(xsr, is);
                return xsr;
            }
            catch (IllegalAccessException e) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e });
            }
            catch (InvocationTargetException e2) {
                Throwable cause = e2.getCause();
                if (cause == null) {
                    cause = e2;
                }
                throw new XMLReaderException("stax.cantCreate", new Object[] { cause });
            }
            catch (XMLStreamException e3) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e3 });
            }
        }
        
        private void reuse(XMLStreamReader xsr, InputSource in) throws IllegalAccessException, InvocationTargetException {
            this.resetMethod.invoke(xsr, new Object[0]);
            this.setInputSourceMethod.invoke(xsr, new Object[] { in });
        }
    }
    
    public static final class Default extends XMLStreamReaderFactory
    {
        private final ThreadLocal<XMLInputFactory> xif;
        
        public Default() {
            this.xif = new ThreadLocal<XMLInputFactory>() {
                public XMLInputFactory initialValue() {
                    return XMLStreamReaderFactory.access$000();
                }
            };
        }
        
        @Override
        public XMLStreamReader doCreate(String systemId, InputStream in, boolean rejectDTDs) {
            try {
                return ((XMLInputFactory)this.xif.get()).createXMLStreamReader(systemId, in);
            }
            catch (XMLStreamException e) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e });
            }
        }
        
        @Override
        public XMLStreamReader doCreate(String systemId, Reader in, boolean rejectDTDs) {
            try {
                return ((XMLInputFactory)this.xif.get()).createXMLStreamReader(systemId, in);
            }
            catch (XMLStreamException e) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e });
            }
        }
        
        @Override
        public void doRecycle(XMLStreamReader r) {
        }
    }
    
    public static class NoLock extends XMLStreamReaderFactory
    {
        private final XMLInputFactory xif;
        
        public NoLock(XMLInputFactory xif) {
            this.xif = xif;
        }
        
        @Override
        public XMLStreamReader doCreate(String systemId, InputStream in, boolean rejectDTDs) {
            try {
                return this.xif.createXMLStreamReader(systemId, in);
            }
            catch (XMLStreamException e) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e });
            }
        }
        
        @Override
        public XMLStreamReader doCreate(String systemId, Reader in, boolean rejectDTDs) {
            try {
                return this.xif.createXMLStreamReader(systemId, in);
            }
            catch (XMLStreamException e) {
                throw new XMLReaderException("stax.cantCreate", new Object[] { e });
            }
        }
        
        @Override
        public void doRecycle(XMLStreamReader r) {
        }
    }
    
    public static final class Woodstox extends XMLStreamReaderFactory.NoLock
    {
        public static final String PROPERTY_MAX_ATTRIBUTES_PER_ELEMENT = "xml.ws.maximum.AttributesPerElement";
        public static final String PROPERTY_MAX_ATTRIBUTE_SIZE = "xml.ws.maximum.AttributeSize";
        public static final String PROPERTY_MAX_CHILDREN_PER_ELEMENT = "xml.ws.maximum.ChildrenPerElement";
        public static final String PROPERTY_MAX_ELEMENT_COUNT = "xml.ws.maximum.ElementCount";
        public static final String PROPERTY_MAX_ELEMENT_DEPTH = "xml.ws.maximum.ElementDepth";
        public static final String PROPERTY_MAX_CHARACTERS = "xml.ws.maximum.Characters";
        private static final int DEFAULT_MAX_ATTRIBUTES_PER_ELEMENT = 500;
        private static final int DEFAULT_MAX_ATTRIBUTE_SIZE = 524288;
        private static final int DEFAULT_MAX_CHILDREN_PER_ELEMENT = Integer.MAX_VALUE;
        private static final int DEFAULT_MAX_ELEMENT_DEPTH = 500;
        private static final long DEFAULT_MAX_ELEMENT_COUNT = 2147483647L;
        private static final long DEFAULT_MAX_CHARACTERS = Long.MAX_VALUE;
        private int maxAttributesPerElement;
        private int maxAttributeSize;
        private int maxChildrenPerElement;
        private int maxElementDepth;
        private long maxElementCount;
        private long maxCharacters;
        private static final String P_MAX_ATTRIBUTES_PER_ELEMENT = "com.ctc.wstx.maxAttributesPerElement";
        private static final String P_MAX_ATTRIBUTE_SIZE = "com.ctc.wstx.maxAttributeSize";
        private static final String P_MAX_CHILDREN_PER_ELEMENT = "com.ctc.wstx.maxChildrenPerElement";
        private static final String P_MAX_ELEMENT_COUNT = "com.ctc.wstx.maxElementCount";
        private static final String P_MAX_ELEMENT_DEPTH = "com.ctc.wstx.maxElementDepth";
        private static final String P_MAX_CHARACTERS = "com.ctc.wstx.maxCharacters";
        private static final String P_INTERN_NSURIS = "org.codehaus.stax2.internNsUris";
        
        public Woodstox(XMLInputFactory xif) {
            super(xif);
            this.maxAttributesPerElement = 500;
            this.maxAttributeSize = 524288;
            this.maxChildrenPerElement = Integer.MAX_VALUE;
            this.maxElementDepth = 500;
            this.maxElementCount = 2147483647L;
            this.maxCharacters = Long.MAX_VALUE;
            if (xif.isPropertySupported("org.codehaus.stax2.internNsUris")) {
                xif.setProperty("org.codehaus.stax2.internNsUris", Boolean.valueOf(true));
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.FINE, "org.codehaus.stax2.internNsUris is {0}", Boolean.valueOf(true));
                }
            }
            if (xif.isPropertySupported("com.ctc.wstx.maxAttributesPerElement")) {
                this.maxAttributesPerElement = (int)Integer.valueOf(XMLStreamReaderFactory.access$300("xml.ws.maximum.AttributesPerElement", 500));
                xif.setProperty("com.ctc.wstx.maxAttributesPerElement", Integer.valueOf(this.maxAttributesPerElement));
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.FINE, "com.ctc.wstx.maxAttributesPerElement is {0}", Integer.valueOf(this.maxAttributesPerElement));
                }
            }
            if (xif.isPropertySupported("com.ctc.wstx.maxAttributeSize")) {
                this.maxAttributeSize = (int)Integer.valueOf(XMLStreamReaderFactory.access$300("xml.ws.maximum.AttributeSize", 524288));
                xif.setProperty("com.ctc.wstx.maxAttributeSize", Integer.valueOf(this.maxAttributeSize));
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.FINE, "com.ctc.wstx.maxAttributeSize is {0}", Integer.valueOf(this.maxAttributeSize));
                }
            }
            if (xif.isPropertySupported("com.ctc.wstx.maxChildrenPerElement")) {
                this.maxChildrenPerElement = (int)Integer.valueOf(XMLStreamReaderFactory.access$300("xml.ws.maximum.ChildrenPerElement", Integer.MAX_VALUE));
                xif.setProperty("com.ctc.wstx.maxChildrenPerElement", Integer.valueOf(this.maxChildrenPerElement));
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.FINE, "com.ctc.wstx.maxChildrenPerElement is {0}", Integer.valueOf(this.maxChildrenPerElement));
                }
            }
            if (xif.isPropertySupported("com.ctc.wstx.maxElementDepth")) {
                this.maxElementDepth = (int)Integer.valueOf(XMLStreamReaderFactory.access$300("xml.ws.maximum.ElementDepth", 500));
                xif.setProperty("com.ctc.wstx.maxElementDepth", Integer.valueOf(this.maxElementDepth));
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.FINE, "com.ctc.wstx.maxElementDepth is {0}", Integer.valueOf(this.maxElementDepth));
                }
            }
            if (xif.isPropertySupported("com.ctc.wstx.maxElementCount")) {
                this.maxElementCount = (long)Long.valueOf(XMLStreamReaderFactory.access$400("xml.ws.maximum.ElementCount", 2147483647L));
                xif.setProperty("com.ctc.wstx.maxElementCount", Long.valueOf(this.maxElementCount));
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.FINE, "com.ctc.wstx.maxElementCount is {0}", Long.valueOf(this.maxElementCount));
                }
            }
            if (xif.isPropertySupported("com.ctc.wstx.maxCharacters")) {
                this.maxCharacters = (long)Long.valueOf(XMLStreamReaderFactory.access$400("xml.ws.maximum.Characters", Long.MAX_VALUE));
                xif.setProperty("com.ctc.wstx.maxCharacters", Long.valueOf(this.maxCharacters));
                if (XMLStreamReaderFactory.LOGGER.isLoggable(Level.FINE)) {
                    XMLStreamReaderFactory.LOGGER.log(Level.FINE, "com.ctc.wstx.maxCharacters is {0}", Long.valueOf(this.maxCharacters));
                }
            }
        }
        
        @Override
        public XMLStreamReader doCreate(String systemId, InputStream in, boolean rejectDTDs) {
            return super.doCreate(systemId, in, rejectDTDs);
        }
        
        @Override
        public XMLStreamReader doCreate(String systemId, Reader in, boolean rejectDTDs) {
            return super.doCreate(systemId, in, rejectDTDs);
        }
    }
    
    public interface RecycleAware
    {
        public abstract void onRecycled();
    }
}
